// mozileKeyboard.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Keyboard Javascript
Mozile version 0.6.16

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-11-16

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, 
and mozileTB.js and separated into different files. The majority
comes from mozilekb.js and mozileTBKeys.js.


DESCRIPTION:
	This file is responsible for handling the keyboard events.

*********/

/*  Older Notes
 *
 *
 * POST04:
 * - reimplement ip use: reuse ip whereever possible. Big performance gain.
 * - support keyboard shortcuts for navigation and style settings
 * - consider xbl equivalent
 * - make sure event handlers aren't loaded twice: if user includes script twice, should
 * not register handlers twice (spotted by Chris McCormick)
 * - see if can move to using DOM events and away from mozileSelection if possible
 * (effects how generic it can be!)
 * - selection model: word, line etc. Write custom handlers of clicks and use new Range
 * expansion methods
 */

/*
 * Handle key presses
 *
 * POST04:
 * - IP isn't recreated everytime with its own text pointer; text pointer isn't (in Range
 * create) set up for every key press.
 * - need up and down arrows to be implemented here too (via eDOM!): that way, no problem with
 * not deselecting toolbar at right time
 * - add in support for typical editing shortcuts based on use of ctrl key or tabs; can synthesize events to
 * force selection. http://www.mozilla.org/docs/end-user/moz_shortcuts.html and ctrl-v seems to effect caret mode?
 * - arrow keys: mode concept where if in text mode then only traverse text AND do not traverse objects. If
 * mixed mode, then select objects too.
 * - each editable area gets a CP? If valid (add method that checks TextNode validity?)
 */





/********* Handler *********/


// Listen for keypresses, and decide what to do
document.addEventListener("keypress", mozileKeys, true);



// Mozile Keys
// Handles keyboard events, counts keys so mozileDo("Typing") can be called

function mozileKeys(event) {

	// Putting the toolbar handling code here is new in 0.611. I hope it doesn't slow things down

		// set the toolbar and autohide variables
	if(mozileToolbar=="") {
		mozileToolbar = document.getElementById("mozile-toolbar");
	}
	
	if(mozileAutohide=="") {
		mozileAutohide = mozilePrefs.getBoolPref('mozile.autohideBool');		// the Mozilla preference which enables autohide
	}
	
	if(mozileCaret=="") {
		mozileCaret = mozilePrefs.getBoolPref('mozile.caretAlwaysOnBool');	// the Mozilla preference which enables caret browsing by default
	}

		// if Mozile is detected, then show the toolbar if it is hidden, and turn the caret on
	if(mozileDetect()) {
		if(mozileToolbar.collapsed) {
			mozileToolbar.collapsed = !mozileToolbar.collapsed;		// show toolbar
		}
		mozilePrefs.setBoolPref('accessibility.browsewithcaret', true);
	}
		// if Mozile is not detected, then hide the toolbar (if autohide is on) and restore the caret state.
	else {
		if(!mozileToolbar.collapsed && mozileAutohide)	{
			mozileToolbar.collapsed = !mozileToolbar.collapsed;		// hide toolbar
		}
		mozilePrefs.setBoolPref('accessibility.browsewithcaret', mozileCaret);		// restore the default caret state
		
		// and don't capture any keys
		return true;
	}
	
	var handled = false;
	var ignoreKeys = new Array(37,38,39,40);		// ignore the arrow keys

	//window.status = event.keyCode;

		// metakey is Mac OSX standard
	if(event.ctrlKey || event.metaKey) {
			// Handle control characters
		handled = ctrlKeyPressHandler(event);
	}

		// handle non-control characters
	else {
			// we want to call mozileDo after [keyMax] characters are entered, so typing can be undone.
			// first makes sure this isn't an arrow key
		var ignore=false;
		for(var i=0; i<ignoreKeys.length; i++) {
			if(event.keyCode==ignoreKeys[i]) {
				ignore=true;
				break;
			}
		}

			// then call mozileDo if the keyCount exceeds the keyMax, otherwise increment the keyCount
		if(!ignore) {
				// create the keycount if needed, and save the current state.
			if(!("keyCount" in mozileTarget) || mozileTarget.keyCount>=keyMax) {
				mozileDo("Typing");
				mozileTarget.keyCount=1;
			}
			else {
				mozileTarget.keyCount++;
			}
		}

			// now handle the keystroke
		handled = nonctrlKeyPressHandler(event);
	}


	// If the event has been handled don't let things go any further.
	if(handled)
	{
		//cancel event: TODO05: why all three?
		event.stopPropagation();
		event.returnValue = false;
  		event.preventDefault();
  		return false;
	}

} // end mozileKeys()







function ctrlKeyPressHandler(event, cssr)
{
	if(!event.charCode)
		return false;

	if(String.fromCharCode(event.charCode).toLowerCase() == "v")
	{
		return mozilePaste();
	}
	else if(String.fromCharCode(event.charCode).toLowerCase() == "x")
	{
		return mozileCut();
	}
	else if(String.fromCharCode(event.charCode).toLowerCase() == "c")
	{
		return mozileCopy();
	}
	else if(String.fromCharCode(event.charCode) == "z")
	{
		return mozileUndo();
	}
	else if(String.fromCharCode(event.charCode) == "Z")
	{
		return mozileRedo();
	}
	else if(String.fromCharCode(event.charCode).toLowerCase() == "s")
	{
		mozileSave();
		return true;
	}
	return false;
}

/**
 * POST04:
 * - carefully move selectEditableRange in here
 * - distinguish editable range of deleteOne at start of line and deleteOne
 * on same line [need to stop merge but allow character deletion]. Perhaps
 * need to change eDOM granularity.
 */
function nonctrlKeyPressHandler(event)
{

	// BACKSPACE (DOM_VK_BACK_SPACE)
	if(event.keyCode == 8)
	{
		return mozileSelection.deleteSelection();
	}
	
		// DELETE (DOM_VK_DELETE)
	if(event.keyCode == 46)
	{
			// this is a hack...
		if(mozileSelection=="") {
			mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_FORWARD_ONE);
		}
		return mozileSelection.deleteSelection();
			
	}
	
	var result;
	// PREV (event.DOM_VK_LEFT) Required as Moz left/right doesn't handle white space properly
	if(event.keyCode == 37 && !event.shiftKey)
	{
		result = mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_BACK_ONE);
		mozileDetect();	// updates the context
		return result;
	}

	// NEXT (event.DOM_VK_RIGHT) Required as Moz left/right doesn't handle white space properly
	if(event.keyCode == 39 && !event.shiftKey)
	{	
		result = mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_FORWARD_ONE);
		mozileDetect();	// updates the context
		return result;
	}

	if(event.keyCode == 38 && !event.shiftKey)
	{
		result = mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_UP_ONE);
		mozileDetect();	// updates the context
		return result;
	}

	if(event.keyCode == 40 && !event.shiftKey)
	{
		result = mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_DOWN_ONE);
		mozileDetect();	// updates the context
		return result;
	}

	// RETURN OR ENTER (event.DOM_VK_ENTER DOM_VK_RETURN)
	if(event.keyCode == 13)
	{
		return mozileSelection.splitXHTMLLine(event.shiftKey);
	}

	// POST05: for non-pre, may change to mean switch to next editable area
	if(event.keyCode == event.DOM_VK_TAB)
	{
		return mozileSelection.insertCharacter(mozileTarget.CHARCODE_SPACE);
	}
	
	var cssr;
	
	// HOME (goes to start of line (block element))
	if(event.keyCode == event.DOM_VK_HOME) {
		cssr = mozileSelection.getEditableRange();

		if(!cssr )
		{
			return false;
		}
		var startip = cssr.lines[0].firstInsertionPoint;
		mozileSelection.collapse(startip.ipNode, startip.ipOffset);
		return true;
	}
	
	// END (goes to end of line (= block element))
	if (event.keyCode == event.DOM_VK_END) {
		cssr = mozileSelection.getEditableRange();
		if(!cssr )
		{
			return false;
		}
		var endip = cssr.lines[0].lastInsertionPoint; 
		mozileSelection.collapse(endip.ipNode, endip.ipOffset);
		return true;
	}

	if(event.keyCode == event.DOM_VK_HOME) 
	{
        	cssr = mozileSelection.getEditableRange();
        	if(!cssr )
        	{
            		return false;
        	}
        	mozileSelection.collapse( cssr.lines[0].container, 0);
	        return true;
    	}

  if (event.keyCode == event.DOM_VK_END)
	{
        	cssr = mozileSelection.getEditableRange();
        	if(!cssr )
        	{
            		return false;
        	}
        	mozileSelection.collapse(cssr.lines[0].container, cssr.lines[0].container.childNodes.length);
        	return true;
	}

	// ALPHANUM
	if(event.charCode) {
		return mozileSelection.insertCharacter(event.charCode);
	}

/*
	// ALPHANUM
		// handle basic ASCII
	if(event.charCode <= 255) {
		window.status = event.charCode;
		return mozileSelection.insertCharacter(event.charCode);
	}
		// insert entities?
	if(event.charCode) {
		var text = "\u"+event.charCode;
		window.status = text;
		return mozileSelection.insertNode(mozileTarget.document.createDocumentFragment(text));
	}
*/


	return false;
}
